@@ -1,6 +1,3 @@
-"""
-:requires: sklearn
-"""
 try:
     from wizzi_utils.algorithms.algorithms import *
 except ModuleNotFoundError as e:
