@@ -1,6 +1,3 @@
-"""
-:sub_package: wizzi_utils.pyplot
-"""
 try:
     from wizzi_utils.algorithms.test.test_algorithms import *
 except ModuleNotFoundError as e:
