@@ -1,6 +1,3 @@
-"""
-:requires: pydrive
-"""
 try:
     from wizzi_utils.google.google_tools import *
 except ModuleNotFoundError as e:
