@@ -1,6 +1,3 @@
-"""
-:sub_package: wizzi_utils.socket
-"""
 try:
     from wizzi_utils.google.test.test_google_tools import *
 except ModuleNotFoundError as e:
