@@ -20,6 +20,8 @@ import glob
 from pathlib import Path
 from uuid import getnode as get_mac
 from traceback import TracebackException
+# noinspection PyProtectedMember
+from pip import _internal
 
 LINES = '-' * 80
 NOT_FOUND = '{} Not found'
@@ -206,6 +208,8 @@ def get_time_stamp(format_s: str = '%Y_%m_%d_%H_%M_%S', ack: bool = False, tabs:
     the default is for files
     :return:
     see get_current_date_hour_test()
+
+    '%Y-%m-%d %H:%M:%S.%f' with ms
     """
     now = datetime.datetime.now()
     time_stamp = now.strftime(format_s)
@@ -249,6 +253,31 @@ def get_mac_address(with_colons: bool = True, ack: bool = False, tabs: int = 1)
     return mac
 
 
+def get_cudnn_version(cudnn_path: str) -> str:
+    cudnn_v = None
+    if os.path.exists(cudnn_path):
+        # extract CUDNN_MAJOR, CUDNN_MINOR, CUDNN_PATCHLEVEL
+        f_obj = open(cudnn_path, 'r')
+        ma, mi, pa = None, None, None
+        for line in f_obj:
+            ls = line.strip()
+            if ls.startswith('#define CUDNN_MAJOR'):
+                ls_spl = ls.split(' ')
+                if len(ls_spl) == 3:
+                    ma = ls_spl[2]
+            elif ls.startswith('#define CUDNN_MINOR'):
+                ls_spl = ls.split(' ')
+                if len(ls_spl) == 3:
+                    mi = ls_spl[2]
+            elif ls.startswith('#define CUDNN_PATCHLEVEL'):
+                ls_spl = ls.split(' ')
+                if len(ls_spl) == 3:
+                    pa = ls_spl[2]
+        if ma and mi and pa:
+            cudnn_v = '{}.{}.{}'.format(ma, mi, pa)
+    return cudnn_v
+
+
 def get_cuda_version(ack: bool = False, tabs: int = 1) -> str:
     """
     :param ack:
@@ -256,11 +285,43 @@ def get_cuda_version(ack: bool = False, tabs: int = 1) -> str:
     :return: cuda version if found on environment variables
     see get_cuda_version_test()
     """
-    cuda_path = get_env_variable(key='CUDA_PATH')
-    if cuda_path is not None:
-        cuda_v = add_color('{}* CUDA Version: {}'.format(tabs * '\t', os.path.basename(cuda_path)), ops=SUCCESS_C)
-    else:
+    cuda_v = None
+    cudnn_v = None
+
+    if is_windows():
+        cuda_path = get_env_variable(key='CUDA_PATH')
+        if cuda_path is not None:
+            cuda_v = os.path.basename(cuda_path)
+            cudnn_v = get_cudnn_version(cudnn_path='{}/include/cudnn.h'.format(cuda_path))
+
+    elif is_linux():
+        cuda_path = get_env_variable(key='CUDA_ROOT')
+        cuda_v_file = '{}/version.txt'.format(cuda_path)
+        if os.path.exists(cuda_v_file):
+            f_obj = open(cuda_v_file, 'r')
+            for line in f_obj:
+                ls = line.strip()
+                if ls.startswith('CUDA Version'):
+                    ls_spl = ls.split(' ')
+                    if len(ls_spl) == 3:
+                        cuda_v = line.strip().split(' ')[2]
+                        break
+        if cuda_v is not None:
+            cudnn_path = '{}/include/cudnn.h'.format(cuda_path)
+            if is_jetson_nano():  # cudnn location on jetson_nano
+                cudnn_path = '/usr/include/aarch64-linux-gnu/cudnn_version_v8.h'
+            cudnn_v = get_cudnn_version(cudnn_path=cudnn_path)
+
+    if cuda_v is None:
         cuda_v = add_color('* No CUDA_PATH found', ops=FAIL_C)
+    else:
+        cuda_v = '{}* CUDA Version: {}'.format(tabs * '\t', cuda_v)
+        if cudnn_v is not None:
+            cuda_v += ' (cuDNN Version {})'.format(cudnn_v)
+        else:
+            cuda_v += ' (No cuDNN found)'
+        cuda_v = add_color(cuda_v, ops=SUCCESS_C)
+
     if ack:
         print(cuda_v)
     return cuda_v
@@ -408,20 +469,11 @@ def version() -> str:
     :return:
     """
     # v = pkg_resources.require("wizzi_utils")[0].version
-    v = '6.6.14'
+    v = '7.0.10'
     return v
 
 
-def is_macos():
-    """
-    :return:
-    see os_test()
-    TODO *** NOT TESTED
-    """
-    return platform.system() == "Darwin"
-
-
-def is_windows():
+def is_windows() -> bool:
     """
     :return:
     see os_test() - tested
@@ -429,7 +481,7 @@ def is_windows():
     return platform.system() == "Windows"
 
 
-def is_linux():
+def is_linux() -> bool:
     """
     :return:
     see os_test()
@@ -437,7 +489,7 @@ def is_linux():
     return platform.system() == "Linux"
 
 
-def is_raspi_arm():
+def is_armv7l() -> bool:
     """
     armv7l is 32 bit processor.
     :return:
@@ -450,13 +502,21 @@ def is_raspi_arm():
     return res
 
 
-def is_aarch64():
+def is_raspberry_pi() -> bool:
+    """
+    armv7l is 32 bit processor.
+    :return:
+    see os_test()
+    """
+    res = is_linux() and is_armv7l()
+    return res
+
+
+def is_aarch64() -> bool:
     """
     ARM64 is the 64-bit extension of the ARM architecture
     :return:
     see os_test()
-    TODO *** NOT TESTED
-    TODO if doesn't work - https://en.wikipedia.org/wiki/Uname
     """
     try:
         res = os.uname()[4] == "aarch64"
@@ -465,16 +525,12 @@ def is_aarch64():
     return res
 
 
-def is_nano():
+def is_jetson_nano() -> bool:
     """
     :return:
-    TODO *** NOT TESTED
     see os_test()
     """
-    try:
-        res = os.uname()[1] == "nano"
-    except AttributeError:
-        res = False
+    res = is_linux() and is_aarch64()
     return res
 
 
@@ -517,6 +573,7 @@ def main_wrapper(
         torch_v: bool = False,
         tf_v: bool = False,
         cv2_v: bool = False,
+        with_pip_list: bool = False,
         with_profiler: bool = False
 ) -> None:
     """
@@ -527,6 +584,7 @@ def main_wrapper(
     :param torch_v: print torch version
     :param tf_v: print tensorflow version
     :param cv2_v: print opencv version
+    :param with_pip_list: print pip list (all libraries installed)
     :param with_profiler: run profiler
     template:
     wu.main_wrapper(
@@ -561,7 +619,7 @@ def main_wrapper(
         try:
             from wizzi_utils.socket.socket_tools import get_ipv4
             print('* Computer ipv4: {}'.format(get_ipv4()))
-        except (ImportError, ModuleNotFoundError, NameError) as err:
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
             print(add_color('* {}'.format(err), 'r'))
 
     cuda_msg = get_cuda_version(ack=False, tabs=0)
@@ -574,35 +632,37 @@ def main_wrapper(
         try:
             from wizzi_utils.torch.torch_tools import get_torch_version
             print(get_torch_version(ack=False, tabs=0))
-        except (ImportError, ModuleNotFoundError, NameError) as err:
-            print(add_color('* {}'.format(err), 'r'))
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+            print(add_color('* torch not Found. error: {}'.format(err), 'r'))
 
     if cv2_v:
         try:
             from wizzi_utils.open_cv.open_cv_tools import get_cv_version
             print(get_cv_version(ack=False, tabs=0))
-        except (ImportError, ModuleNotFoundError, NameError) as err:
-            pip = 'pip install opencv-python'
-            print(add_color('* {} - {}'.format(err, pip), 'r'))
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+            print(add_color('* opencv-python not Found. error: {}'.format(err), 'r'))
 
     if tf_v:
-        try:
-            from wizzi_utils.tensorflow.tensorflow_tools import get_tensorflow_version
-            print(get_tensorflow_version(ack=False, tabs=0))
-        except (ImportError, ModuleNotFoundError, NameError) as err:
-            print(add_color('* {}'.format(err), 'r'))
+        # try:
+        #     from wizzi_utils.tensorflow.tensorflow_tools import get_tensorflow_version
+        #     print(get_tensorflow_version(ack=False, tabs=0))
+        # except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+        #     print(add_color('* tensorflow not Found. error: {}'.format(err), 'r'))
 
         try:
             from wizzi_utils.tflite.tflite_tools import get_tflite_version
             print(get_tflite_version(ack=False, tabs=0))
-        except (ImportError, ModuleNotFoundError, NameError) as err:
-            pip = '\n\twin os: pip install --extra-index-url https://google-coral.github.io/py-repo/ tflite_runtime'
-            print(add_color('* {} - {}'.format(err, pip), 'r'))
+        except (ImportError, ModuleNotFoundError, NameError, AttributeError) as err:
+            print(add_color('* tflite_runtime not Found. error: {}'.format(err), 'r'))
 
     if seed > -1:
         set_seed(seed=seed)
         print('* Seed was initialized to {}'.format(seed))
 
+    if with_pip_list:
+        print('* Pip list output:')
+        _internal.main(['list'])
+
     print('Function {} started:'.format(main_function))
     print(LINES)
     pr = start_profiler() if with_profiler else None
@@ -726,10 +786,17 @@ def to_str(var: any,
 
         if chars != 0:  # -1 for all, or x>0 for x chars
             if fp >= 0:
-                if is_list(var):
+                if all((is_int(item) or is_float(item)) for item in var):  # 1d list of int and floats
+                    if all(is_int(item) for item in var):  # if all ints - no rounding
+                        f_format = '{:,}'
+                    else:
+                        f_format = '{:,.%xf}' % fp
+                    new_v = [f_format.format(li_item) for li_item in var]
+                else:  # >1d list or 1d with not just ints and floats
                     new_v = round_list(var, fp=fp)
-                else:
-                    new_v = round_tuple(var, fp=fp)
+
+                if is_tuple(var):
+                    new_v = tuple(new_v)
             else:
                 new_v = var
             data_str_raw = str(new_v).replace('\'', '')
@@ -746,11 +813,18 @@ def to_str(var: any,
             string += ',s={}'.format(var.shape)
             string += ',dtype={})'.format(var.dtype)
         if chars != 0:  # -1 for all, or x>0 for x chars
+            # new_v = var.tolist()
             if fp >= 0:
-                new_v = round_list(var.tolist(), fp=fp)
+                if all((is_int(item) or is_float(item)) for item in var):  # 1d list of int and floats
+                    if all(is_int(item) for item in var):  # if all ints - no rounding
+                        f_format = '{:,}'
+                    else:
+                        f_format = '{:,.%xf}' % fp
+                    new_v = [f_format.format(li_item) for li_item in var]
+                else:  # >1d list or 1d with not just ints and floats
+                    new_v = np.around(var.tolist(), fp).tolist()
             else:
-                new_v = var.tolist()
-
+                new_v = var
             data_str_raw = str(new_v).replace('\'', '')
             string += get_data_str(data_str_raw, chars)
 
@@ -1524,7 +1598,7 @@ def round_tuple(t: tuple, fp: int = 3, warn: bool = False):
     return new_t
 
 
-def round_list(li: (list, tuple), fp: int = 3, warn: bool = False):
+def round_list(li: (list, tuple), fp: int = 3, warn: bool = False) -> list:
     """
     :param li: list or tuple 1D of floats
     :param fp: float precision >= 0
@@ -1535,21 +1609,14 @@ def round_list(li: (list, tuple), fp: int = 3, warn: bool = False):
 
     new_li = li
     if fp >= 0 and len(li) > 0:
-        if all((is_int(item) or is_float(item)) for item in li):  # 1d list of int and floats
-            if all(is_int(item) for item in li):  # if all ints - no rounding
-                f_format = '{:,}'
-            else:
-                f_format = '{:,.%xf}' % fp
-            new_li = [f_format.format(li_item) for li_item in li]
-        else:  # >1d list or 1d with not just ints and floats
-            try:
-                new_np = np.array(li)
-                # if >1d list of floats\ints - this will work
-                if new_np.dtype in [float, np.float, np.float32, np.float64]:
-                    new_li = np.around(new_np, fp).tolist()
-            except TypeError as e:  # list has non ints\floats elements or uneven size ...
-                if warn:
-                    exception_error(e, real_exception=True)
+        try:
+            new_np = np.array(li)
+            # if >1d list of floats\ints - this will work
+            if new_np.dtype in [float, np.float, np.float32, np.float64]:
+                new_li = np.around(new_np, fp).tolist()
+        except TypeError as e:  # list has non ints\floats elements or uneven size ...
+            if warn:
+                exception_error(e, real_exception=True)
     return new_li
 
 
@@ -1857,15 +1924,15 @@ def wizzi_utils_requirements():
     sub_pack = 'Sub Package: {}'
 
     MT_PACK, MT_MOD, MT_SC = 'misc', 'misc_tools', 'mt'
-    ALG_PACK, ALG_MOD, ALG_SC = 'algorithms', 'algorithms', 'alg'
-    COT_PACK, COT_MOD, COT_SC = 'coreset', 'coreset_tools', 'cot'
+    # ALG_PACK, ALG_MOD, ALG_SC = 'algorithms', 'algorithms', 'alg'
+    # COT_PACK, COT_MOD, COT_SC = 'coreset', 'coreset_tools', 'cot'
     GOT_PACK, GOT_MOD, GOT_SC = 'google', 'google_tools', 'got'
     JT_PACK, JT_MOD, JT_SC = 'json', 'json_tools', 'jt'
     CVT_PACK, CVT_MOD, CVT_SC = 'open_cv', 'open_cv_tools', 'cvt'
     PYPLT_PACK, PYPLT_MOD, PYPLT_SC = 'pyplot', 'pyplot_tools', 'pyplt'
     ST_PACK, ST_MOD, ST_SC = 'socket', 'socket_tools', 'st'
     TT_PACK, TT_MOD, TT_SC = 'torch', 'torch_tools', 'tt'
-    TFT_PACK, TFT_MOD, TFT_SC = 'tensorflow', 'tensorflow_tools', 'tft'
+    # TFT_PACK, TFT_MOD, TFT_SC = 'tensorflow', 'tensorflow_tools', 'tft'
     TFLT_PACK, TFLT_MOD, TFLT_SC = 'tflite', 'tflite_tools', 'tflt'
     TTS_PACK, TTS_MOD, TTS_SC = 'tts', 'tts', 'tts'
     MODELS_PACK, MODELS_MOD, MODELS_SC = 'models', 'models', 'models'
@@ -1878,17 +1945,17 @@ def wizzi_utils_requirements():
     print(pack_test.format(MT_MOD, MT_SC))
     print('\t\tNone')
 
-    print(LINES)
-    print(pack.format(ALG_PACK, ALG_MOD, ALG_SC))
-    print('\t{}'.format(pip.format('sklrean')))
-    print(pack_test.format(ALG_MOD, ALG_SC))
-    print('\t\t{}'.format(sub_pack.format(PYPLT_PACK)))
+    # print(LINES)
+    # print(pack.format(ALG_PACK, ALG_MOD, ALG_SC))
+    # print('\t{}'.format(pip.format('sklrean')))
+    # print(pack_test.format(ALG_MOD, ALG_SC))
+    # print('\t\t{}'.format(sub_pack.format(PYPLT_PACK)))
 
-    print(LINES)
-    print(pack.format(COT_PACK, COT_MOD, COT_SC))
-    print('\tNone')
-    print(pack_test.format(COT_MOD, COT_SC))
-    print('\t\tNone')
+    # print(LINES)
+    # print(pack.format(COT_PACK, COT_MOD, COT_SC))
+    # print('\tNone')
+    # print(pack_test.format(COT_MOD, COT_SC))
+    # print('\t\tNone')
 
     print(LINES)
     print(pack.format(GOT_PACK, GOT_MOD, GOT_SC))
@@ -1908,21 +1975,17 @@ def wizzi_utils_requirements():
     print(pack.format(CVT_PACK, CVT_MOD, CVT_SC))
     print('\t{}'.format(pip.format('opencv-python')))
     print('\t{}'.format(sub_pack.format(PYPLT_PACK)))
-    print('\t{}'.format(sub_pack.format(ST_PACK)))
     print('\t{} # for CameraWu type_cam=="imutils"'.format(pip.format('imutils')))
     print('\t{} # for CameraWu type_cam=="acapture"'.format(pip.format('acapture')))
     print(pack_test.format(CVT_MOD, CVT_SC))
     print('\t\t{}'.format(sub_pack.format(ST_PACK)))
-    print('\t\t{}'.format(sub_pack.format(CVT_PACK)))
 
     print(LINES)
     print(pack.format(PYPLT_PACK, PYPLT_MOD, PYPLT_SC))
-    print('\t{}'.format(pip.format('matplotlib')))
-    print('\t{}'.format(pip.format('mpl_toolkits')))
     print('\t{} # for compare_images_multi_sets_squeezed()'.format(pip.format('torch')))
     print(pack_test.format(PYPLT_MOD, PYPLT_SC))
     print('\t\t{} # for compare_images_multi_sets_squeezed_test()'.format(pip.format('torch')))
-    print('\t\t{} # also the test'.format(sub_pack.format(CVT_PACK)))
+    print('\t\t{} # with the tests as well'.format(sub_pack.format(CVT_PACK)))
 
     print(LINES)
     print(pack.format(ST_PACK, ST_MOD, ST_SC))
@@ -1931,34 +1994,34 @@ def wizzi_utils_requirements():
     print('\t\t{}'.format(sub_pack.format(JT_PACK)))
 
     print(LINES)
-    url = 'https://download.pytorch.org/whl/cu90/torch_stable.html'
+
     print(pack.format(TT_PACK, TT_MOD, TT_SC))
-    print('\ttorch options:')
+    print('\ttorch:')
     print('\t\t{} # cpu'.format(pip.format('torch')))
-    print('\t\t{} # gpu(cuda 9 and 9.1)'.format(pip.format('torch==1.1.0 torchvision==0.3.0 -f {}'.format(url))))
+    print('\t\tfor gpu support check your cuda and google for the pip command')
     print('\t\tall options: https://pytorch.org/get-started/locally/')
     print('\t\t** may need to uninstall numpy and re-install')
     print('\t{}'.format(pip.format('torchvision')))
     print(pack_test.format(TT_MOD, TT_SC))
     print('\t\tNone')
 
-    print(LINES)
-    print(pack.format(TFT_PACK, TFT_MOD, TFT_SC))
-    print('\ttensorflow options:')
-    print('\t\t{} # cpu only'.format(pip.format('tensorflow')))
-    print('\t\t{} # gpu cuda 9.1'.format(pip.format('tensorflow-gpu==1.12')))
-    print('\t\t{} # gpu cuda>=10'.format(pip.format('tensorflow-gpu')))
-    url = 'https://stackoverflow.com/questions/50622525/'
-    url += 'which-tensorflow-and-cuda-version-combinations-are-compatible'
-    print('\t\tall options: {}'.format(url))
-    print(pack_test.format(TFT_MOD, TFT_SC))
-    print('\t\tNone')
+    # print(LINES)
+    # print(pack.format(TFT_PACK, TFT_MOD, TFT_SC))
+    # print('\ttensorflow:')
+    # print('\t\t{}'.format(pip.format('tensorflow')))
+    # url = 'https://stackoverflow.com/questions/50622525/'
+    # url += 'which-tensorflow-and-cuda-version-combinations-are-compatible'
+    # print('\t\tall options: {}'.format(url))
+    # print(pack_test.format(TFT_MOD, TFT_SC))
+    # print('\t\tNone')
 
     print(LINES)
     print(pack.format(TFLT_PACK, TFLT_MOD, TFLT_SC))
     print('\ttflite options:')
-    url = 'https://google-coral.github.io/py-repo/'
-    print('\t\t{} # windows 10'.format(pip.format('--extra-index-url {} tflite_runtime'.format(url))))
+    url_win10 = 'https://google-coral.github.io/py-repo/'
+    print('\t\t{} # windows 10 and linux'.format(pip.format('--index-url {} tflite_runtime'.format(url_win10))))
+    print('\t\t\tmore info on https://www.tensorflow.org/lite/guide/python')
+
     print(pack_test.format(TFLT_MOD, TFLT_SC))
     print('\t\t{}'.format(None))
 
@@ -1973,8 +2036,8 @@ def wizzi_utils_requirements():
     print('\t{}'.format(sub_pack.format(ST_PACK)))
     print('\t{}'.format(sub_pack.format(PYPLT_PACK)))
     print('\t{}'.format(sub_pack.format(CVT_PACK)))
-    print(pack_test.format(MODELS_MOD, MODELS_SC))
-    print('\t\tNone')
+    print('\t{} # for tracking'.format(pip.format('opencv-contrib-python')))
+    print('\tinstall tflite for od and pd models')
     print(LINES)
     return
 
@@ -2142,6 +2205,10 @@ class FPS:
     def __str__(self) -> str:
         return ''
 
+    def _times_elapsed_not_empty(self) -> bool:
+        ret = True if len(self.times_elapsed) > 0 else False
+        return ret
+
     def start(self, ack_progress: bool = False, tabs: int = 1, with_title: bool = False) -> None:
         if ack_progress:
             string = tabs * '\t'
@@ -2167,34 +2234,15 @@ class FPS:
         return
 
     def get_fps(self) -> float:
-        avg_times = np.mean(self.times_elapsed)
-        if avg_times > 0.0001:
-            fps = 1 / avg_times
-        else:
-            fps = -1
+        fps = -1.0
+        if self._times_elapsed_not_empty():
+            avg_times = np.mean(self.times_elapsed)
+            if avg_times > 0.0001:
+                fps = 1 / avg_times
         return fps
 
-    # def convert(self) -> str:
-    #     rt = self.times_elapsed[-1]
-    #     if rt < 0.001:  # 0.0001 -> 0.100ms
-    #         rt_str = '{:.3f}ms'.format(rt * 1000)  # fp=6 digit
-    #     elif rt < 0.1:
-    #         rt_str = '{:.2f}ms'.format(rt * 1000)  # fp=6 digit
-    #     elif rt < 1:
-    #         rt_str = '{:.1f}ms'.format(rt * 1000)
-    #     elif rt < 20:
-    #         rt_str = '{:.3f}s'.format(rt)
-    #     elif rt < 60:
-    #         rt_str = '{:.1f}s'.format(rt)
-    #     elif rt < 120:
-    #         m, s = divmod(rt, 60)
-    #         h, m = divmod(m, 60)
-    #         rt_str = '{}:{:.1f}'.format(int(m), s)
-    #     print('\trt={:.6f}s == {}'.format(rt, rt_str))
-    #     return rt_str
-
     def so_far(self, tabs: int = 1, with_title: bool = False) -> None:
-        if len(self.times_elapsed) > 0:
+        if self._times_elapsed_not_empty():
             string = tabs * '\t'
             if with_title:
                 string += '{}:'.format(self.summary_title)
@@ -2208,6 +2256,7 @@ class FPS:
             if len(self.times_elapsed) > 1:
                 string += 'totalTime={:.3f}s, '.format(np.sum(self.times_elapsed))
                 string += 'avgTime={:.3f}s, '.format(avg_times)
+
             if avg_times > 0.0001:
                 fps = '{:.2f} FPS'.format(1 / avg_times)
             else:
@@ -2216,11 +2265,13 @@ class FPS:
             print(string)
         return
 
+    def clear(self) -> None:
+        self.times_elapsed = []
+        return
+
     def finalize(self, tabs: int = 1) -> None:
         tabs_s = tabs * '\t'
-        if len(self.times_elapsed) <= 0:
-            exception_error('Times_elapsed is empty', real_exception=False, tabs=tabs)
-        else:
+        if self._times_elapsed_not_empty():
             # title
             title = 'Summary '
             if self.summary_title is not None:
@@ -2244,4 +2295,6 @@ class FPS:
             else:
                 fps = '{}\tFPS very high'.format(tabs * '\t')
             print(add_color(fps, ops='r'))
+        else:
+            exception_error('Times_elapsed is empty', real_exception=False, tabs=tabs)
         return
