@@ -22,12 +22,12 @@ JUST_A_NAME = 'just_a_name'
 FAKE_FILE = 'fake_file'
 
 SO_LOGO = 'so_logo'
-KITE = 'kite'
+KITE = 'kite'  # (900, 1352, 3)
 GIRAFFE = 'giraffe'
 HORSES = 'horses'
-DOG = 'dog'
-EAGLE = 'eagle'
-PERSON = 'person'
+DOG = 'dog'  # (576, 768, 3)
+EAGLE = 'eagle'  # (512, 773, 3)
+PERSON = 'person'  # (424, 640, 3)
 DOGS1 = 'dogs1'
 FACES = 'faces'
 F_MODEL = 'model2'
@@ -194,12 +194,14 @@ def profiler_test():
 
 def os_test():
     mt.get_function_name(ack=True, tabs=0)
-    print('\tis_macos     {}'.format(mt.is_macos()))
-    print('\tis_windows   {}'.format(mt.is_windows()))
-    print('\tis_linux     {}'.format(mt.is_linux()))
-    print('\tis_raspi_arm {}'.format(mt.is_raspi_arm()))
-    print('\tis_aarch64   {}'.format(mt.is_aarch64()))
-    print('\tis_nano      {}'.format(mt.is_nano()))
+    print('\tis_windows     {}'.format(mt.is_windows()))
+    print('\tis_linux       {}'.format(mt.is_linux()))
+
+    print('\tis_armv7l      {}'.format(mt.is_armv7l()))
+    print('\tis_raspberrypi {}'.format(mt.is_raspberry_pi()))
+
+    print('\tis_aarch64     {}'.format(mt.is_aarch64()))
+    print('\tis_jetson_nano {}'.format(mt.is_jetson_nano()))
     return
 
 
@@ -844,11 +846,15 @@ def classFPS_test():
     mt.get_function_name_and_line(ack=True, tabs=0)
     fps = mt.FPS(summary_title='classFPS_test')
     tabs = 1
+    print('{}test empty get_fps() = {:.4f}'.format(tabs * '\t', fps.get_fps()))
     for t in range(3):
         fps.start(ack_progress=True, tabs=tabs)
         # do_work of round t
         mt.sleep(seconds=0.03)
         fps.update(ack_progress=True, tabs=tabs + 1)
+        print('{}\tget_fps() = {:.4f}'.format(tabs * '\t', fps.get_fps()))
+        if t == 0:
+            fps.clear()
     fps.finalize(tabs)
     return
 
