@@ -13,9 +13,9 @@ class ModelType(Enum):
 
 class Jobs(Enum):
     OBJECT_DETECTION = 'object_detection'
-    SEGMENTATION = 'segmentation'
+    SEGMENTATION = 'segmentation'  # future
     POSE_DETECTION = 'pose_detection'
-    CLASSIFICATION = 'classification'
+    CLASSIFICATION = 'classification'  # future
 
 
 class DnnFamily(Enum):
