@@ -6,24 +6,18 @@ from enum import Enum
 from mpl_toolkits.mplot3d import Axes3D
 from mpl_toolkits.mplot3d.art3d import Poly3DCollection
 from mpl_toolkits.mplot3d.art3d import Path3DCollection
-from wizzi_utils.misc import misc_tools as mt
-# noinspection PyPackageRequirements
 import matplotlib
-# noinspection PyPackageRequirements
 import matplotlib.pyplot as plt
-# noinspection PyPackageRequirements
 from matplotlib.figure import Figure
-# noinspection PyPackageRequirements
 import matplotlib.axes
-# noinspection PyPackageRequirements
 from matplotlib.backend_bases import KeyEvent
-# noinspection PyPackageRequirements
 from matplotlib.backend_bases import CloseEvent
-# noinspection PyPackageRequirements
 from matplotlib.collections import PathCollection
-# noinspection PyPackageRequirements,PyProtectedMember
+# noinspection PyProtectedMember
 from matplotlib.backends._backend_tk import TimerTk
 
+from wizzi_utils.misc import misc_tools as mt
+
 
 class Location(Enum):
     TOP_LEFT = 'top_left'
@@ -36,6 +30,37 @@ class Location(Enum):
     BOTTOM_CENTER = 'bottom_center'
     BOTTOM_RIGHT = 'bottom_right'
 
+    @staticmethod
+    def where_to(w_short: str) -> str:
+        """
+        instead of writing:
+        wu.pyplt.Location.TOP_LEFT.value
+        write:
+        wu.pyplt.Location.where('tl')
+        :param w_short:
+        :return:
+        """
+        w_long = None
+        if w_short == 'tl':
+            w_long = Location.TOP_LEFT.value
+        elif w_short == 'tc':
+            w_long = Location.TOP_CENTER.value
+        elif w_short == 'tr':
+            w_long = Location.TOP_RIGHT.value
+        elif w_short == 'cl':
+            w_long = Location.CENTER_LEFT.value
+        elif w_short == 'cc':
+            w_long = Location.CENTER_CENTER.value
+        elif w_short == 'cr':
+            w_long = Location.CENTER_RIGHT.value
+        elif w_short == 'bl':
+            w_long = Location.BOTTOM_LEFT.value
+        elif w_short == 'bc':
+            w_long = Location.BOTTOM_CENTER.value
+        elif w_short == 'br':
+            w_long = Location.BOTTOM_RIGHT.value
+        return w_long
+
     @staticmethod
     def get_location_list_by_rows():
         """
@@ -528,48 +553,61 @@ def move_figure_x_y(fig: matplotlib.figure, x_y: tuple) -> None:
     return
 
 
-def move_figure_by_str(fig: matplotlib.figure, where: str = Location.TOP_LEFT.value) -> None:
+def calc_x_y_by_loc_str(
+        where: str,
+        window_w: int, window_h: int,
+        fig_w: int, fig_h: int,
+        task_bar_offset: int = 0
+) -> (int, int):
+    x, y = 0, 0  # Location.TOP_LEFT.value: default
+    if where == Location.TOP_CENTER.value:
+        x = (window_w - fig_w) / 2
+        y = 0
+    elif where == Location.TOP_RIGHT.value:
+        x = window_w - fig_w
+        y = 0
+    elif where == Location.CENTER_LEFT.value:
+        x = 0
+        y = (window_h - fig_h - task_bar_offset) / 2
+    elif where == Location.CENTER_CENTER.value:
+        x = (window_w - fig_w) / 2
+        y = (window_h - fig_h - task_bar_offset) / 2
+    elif where == Location.CENTER_RIGHT.value:
+        x = window_w - fig_w
+        y = (window_h - fig_h - task_bar_offset) / 2
+    elif where == Location.BOTTOM_LEFT.value:
+        x = 0
+        y = window_h - fig_h - task_bar_offset
+    elif where == Location.BOTTOM_CENTER.value:
+        x = (window_w - fig_w) / 2
+        y = window_h - fig_h - task_bar_offset
+    elif where == Location.BOTTOM_RIGHT.value:
+        x = window_w - fig_w
+        y = window_h - fig_h - task_bar_offset
+    x, y = int(x), int(y)
+    return x, y
+
+
+def move_figure_by_str(fig: matplotlib.figure, where: str = Location.TOP_LEFT.value,
+                       task_bar_offset: int = None) -> None:
     """
     :param fig: figure to be moved
     :param where: see Location enum. e.g. Location.TOP_LEFT.value (which is 'top_left')
+    :param task_bar_offset: size of taskbar for bottom locs
     see move_figure_by_str_test()
     """
+    where_full = Location.where_to(w_short=where)
+    if where_full is not None:  # if not None, shortcut was entered
+        where = where_full
     try:
         window_w, window_h = screen_dims()  # screen dims in pixels
         if window_w != -1 and window_h != -1:
             fig_w, fig_h = fig.get_size_inches() * fig.dpi  # fig dims in pixels
-            task_bar_offset = 100  # about 100 pixels due to task bar
-            x, y = 0, 0  # Location.TOP_LEFT.value: default
-            if where == Location.TOP_CENTER.value:
-                x = (window_w - fig_w) / 2
-                y = 0
-            elif where == Location.TOP_RIGHT.value:
-                x = window_w - fig_w
-                y = 0
-            elif where == Location.CENTER_LEFT.value:
-                x = 0
-                y = (window_h - fig_h - task_bar_offset) / 2
-            elif where == Location.CENTER_CENTER.value:
-                x = (window_w - fig_w) / 2
-                y = (window_h - fig_h - task_bar_offset) / 2
-            elif where == Location.CENTER_RIGHT.value:
-                x = window_w - fig_w
-                y = (window_h - fig_h - task_bar_offset) / 2
-            elif where == Location.BOTTOM_LEFT.value:
-                x = 0
-                y = window_h - fig_h - task_bar_offset
-            elif where == Location.BOTTOM_CENTER.value:
-                x = (window_w - fig_w) / 2
-                y = window_h - fig_h - task_bar_offset
-            elif where == Location.BOTTOM_RIGHT.value:
-                x = window_w - fig_w
-                y = window_h - fig_h - task_bar_offset
-            x, y = int(x), int(y)
+            if task_bar_offset is None:
+                # no toolbar 75 is good. with toolbar 100
+                task_bar_offset = 75 if matplotlib.rcParams['toolbar'] == 'none' else 100
+            x, y = calc_x_y_by_loc_str(where, window_w, window_h, fig_w, fig_h, task_bar_offset=task_bar_offset)
             move_figure_x_y(fig=fig, x_y=(x, y))
-            # print('{}'.format(where))
-            # print('\t window: {} {}'.format(window_w, window_h))
-            # print('\t figure: {} {}'.format(fig_w, fig_h))
-            # print('\tx_y: {} {}'.format(x, y))
         else:
             move_figure_x_y(fig=fig, x_y=(0, 0))
     except (ValueError, Exception) as e:
@@ -580,7 +618,11 @@ def move_figure_by_str(fig: matplotlib.figure, where: str = Location.TOP_LEFT.va
 
 def set_window_title(fig: matplotlib.figure, title: str) -> None:
     """ sets window title """
-    fig.canvas.set_window_title(title)
+    # fig.canvas.set_window_title(title)  # deprecated
+    try:
+        fig.canvas.manager.set_window_title(title)
+    except AttributeError:
+        pass
     return
 
 
@@ -691,19 +733,21 @@ def add_timer(fig: matplotlib.figure, max_time_seconds: float) -> TimerTk:
     return timer
 
 
-def finalize_plot(fig: matplotlib.figure, max_time_seconds: float = None) -> None:
+def finalize_plot(fig: matplotlib.figure, max_time_seconds: float = None,
+                  end_str: (str, None) = 'click Esc to close') -> None:
     """
     :param fig:
     :param max_time_seconds: if not None - close plot after max_time_seconds
+    :param end_str: if not None - write end_str at the end of the title
     blocks the plot
     """
-    end_str = 'click Esc to close'
-    # noinspection PyProtectedMember
-    old_title_obj = fig._suptitle
-    if old_title_obj is not None:
-        set_figure_title(fig, '{} - {}'.format(old_title_obj.get_text(), end_str))
-    else:
-        set_figure_title(fig, end_str)
+    if end_str is not None:
+        # noinspection PyProtectedMember
+        old_title_obj = fig._suptitle
+        if old_title_obj is not None:
+            set_figure_title(fig, '{} - {}'.format(old_title_obj.get_text(), end_str))
+        else:
+            set_figure_title(fig, end_str)
 
     fig.canvas.mpl_connect('key_press_event', on_click_close_all)  # close on ESC pressed
     timer = None
@@ -1708,14 +1752,26 @@ def plot_3d_one_dashboard(
     return
 
 
-def plot_3d_cube(axes: Axes3D, cube_definition: list, color='b', label='cube', add_labels=False) -> None:
+def plot_3d_cube(
+        axes: Axes3D,
+        cube_definition: list,
+        label='cube',
+        face_color: str = 'blue',
+        face_opacity: float = 0.1,
+        edge_color: str = 'black',
+        edge_width: float = 1.0,
+        add_corners_coordinates: bool = False,
+) -> None:
     """
     :param axes:
     :param cube_definition: list of np.arrays
         4 points of the cube (1 corner node and 3 nodes to the left right and top)
-    :param color:
     :param label:
-    :param add_labels: add cube corners label on figure
+    :param face_color:
+    :param face_opacity:
+    :param edge_color:
+    :param edge_width:
+    :param add_corners_coordinates: add cube corners label on figure
     see plot_3d_cube_test()
     """
 
@@ -1743,19 +1799,19 @@ def plot_3d_cube(axes: Axes3D, cube_definition: list, color='b', label='cube', a
         [points[3], points[6], points[7], points[5]]
     ]
 
-    faces = Poly3DCollection(edges, linewidths=1, edgecolors='k')
+    faces = Poly3DCollection(edges, linewidths=edge_width, edgecolors=edge_color)
     # noinspection PyUnresolvedReferences
-    color_rgba = matplotlib.colors.to_rgba(color, alpha=0.1)
+    color_rgba = matplotlib.colors.to_rgba(face_color, alpha=face_opacity)
     faces.set_facecolor(color_rgba)
     axes.add_collection3d(faces)
 
     # Plot the points themselves to force the scaling of the axes
-    axes.scatter(points[:, 0], points[:, 1], points[:, 2], s=0.1, color=color, label=label)
+    axes.scatter(points[:, 0], points[:, 1], points[:, 2], s=0.1, color=edge_color, label=label)
 
-    if add_labels:
+    if add_corners_coordinates:
         for p in points:
             x, y, z = p
-            text = '{},{},{}'.format(x, y, z)
+            text = '{:.2f},{:.2f},{:.2f}'.format(x, y, z)
             axes.text(x, y, z, text, zdir=(1, 1, 1))
     return
 
@@ -1782,7 +1838,17 @@ def add_cube3d_around_origin(axes: Axes3D, edge_len: int, color: str = 'b', add_
         axes=axes,
         cube_definition=cube_4_edges,
         label='cube(edge={})'.format(edge_len),
-        color=color,
-        add_labels=add_labels
+        face_color=color,
+        add_corners_coordinates=add_labels
     )
     return
+
+
+def plt_dark_mode() -> None:
+    plt.style.use('dark_background')
+    return
+
+
+def remove_toolbar() -> None:
+    matplotlib.rcParams['toolbar'] = 'None'
+    return
