@@ -1,12 +1,12 @@
+import numpy as np
+from mpl_toolkits.mplot3d import Axes3D
+import matplotlib.pyplot as plt
+
 from wizzi_utils.pyplot import pyplot_tools as pyplt
 from wizzi_utils.misc import misc_tools as mt
 from wizzi_utils.misc.test import test_misc_tools as mtt
 from wizzi_utils.open_cv import open_cv_tools as cvt
 from wizzi_utils.open_cv.test import test_open_cv_tools as cvtt
-import numpy as np
-from mpl_toolkits.mplot3d import Axes3D
-# noinspection PyPackageRequirements
-import matplotlib.pyplot as plt
 
 SLEEP = 0.2  # sleep between iterations
 PLT_MAX_TIME = 2  # close plt after x seconds - minimum 2
@@ -149,6 +149,33 @@ def move_figure_by_str_test():
     return
 
 
+def move_figure_by_short_str_test():
+    mt.get_function_name(ack=True, tabs=0)
+    t = np.arange(0.0, 2.0, 0.01)
+    s = 1 + np.sin(2 * np.pi * t)
+    options = ['tl', 'cc', 'br']
+    print('\tVisual test: move to all options {}'.format(options))
+    print('\t\tClick Esc to close all')
+    timers = []
+    for where_to in options:
+        title = 'move to {}'.format(where_to)
+        resize = 0.7
+        figsize = (6.4 * resize, 4.8 * resize)
+        fig, ax = plt.subplots(figsize=figsize)
+        pyplt.set_window_title(fig, title=title)
+        pyplt.set_figure_title(fig, title=title)
+        pyplt.set_axes_title(ax, title)
+        fig.canvas.mpl_connect('key_press_event', pyplt.on_click_close_all)
+        ax.plot(t, s, c=pyplt.get_random_color('str'))
+        pyplt.move_figure_by_str(fig, where=where_to)
+        timer = pyplt.add_timer(fig, PLT_MAX_TIME)
+        timers.append(timer)
+    plt.show(block=True)
+    for timer in timers:
+        timer.stop()
+    return
+
+
 def add_update_2d_scatter_test():
     mt.get_function_name(ack=True, tabs=0)
     fig, axes = plt.subplots()
@@ -984,8 +1011,12 @@ def plot_3d_cube_test():
     pyplt.plot_3d_cube(
         ax,
         cube_definition=cube_def,
-        color='r',
-        add_labels=True
+        label='my cube',
+        face_color='blue',
+        face_opacity=0.5,
+        edge_color='black',
+        edge_width=3.0,
+        add_corners_coordinates=True,
     )
     legend = pyplt.LEGEND_DEFAULT
     plt.legend(loc=legend['loc'], ncol=legend['ncol'], fancybox=legend['fancybox'],
@@ -1034,6 +1065,7 @@ def test_all():
     screen_dims_test()
     move_figure_x_y_test()
     move_figure_by_str_test()
+    move_figure_by_short_str_test()
 
     # iterative plots - 2d and 3d
     add_update_2d_scatter_test()
