@@ -166,6 +166,23 @@ def download_file_test():
     return
 
 
+def get_file_size_by_url_test():
+    mt.get_function_name(ack=True, tabs=0)
+
+    urls = [
+        mtt.IMAGES_D[mtt.SO_LOGO],
+        mtt.IMAGES_D[mtt.KITE],
+        mtt.IMAGES_D[mtt.GIRAFFE],
+        'https://cdn.sstatic.net/Sites/stackoverflow/img/NoSuchFile.png'
+    ]
+    for url in urls:
+        url_file = url.split('/')[-1]
+        s = st.get_file_size_by_url(url=url)
+        print('\tfile {} size is {}'.format(url_file, s))
+
+    return
+
+
 def test_all():
     print('{}{}:'.format('-' * 5, mt.get_base_file_and_function_name()))
     open_server_test()
@@ -173,5 +190,6 @@ def test_all():
     get_ipv4_test()
     buffer_to_str_test()
     download_file_test()
+    get_file_size_by_url_test()
     print('{}'.format('-' * 20))
     return
