@@ -181,26 +181,37 @@ def to_str(var: any,
     :return: informative string of the variable
     see to_str_test() on test_torch_tools.py
     """
+
+    string = title
+    type_s = str(type(var)).replace('<class \'', '').replace('\'>', '')  # clean type name
+
     if is_torch(var):
-        string = title
         if wm:
-            type_s = str(type(var)).replace('<class \'', '').replace('\'>', '').replace('torch.', '')
             string += '({}'.format(type_s)
-            string += ',s={}'.format(size_s(var))
+            string += ',s={}'.format(var.shape)
             string += ',dtype={}'.format(var.dtype)
             string += ',trainable={}'.format(is_trainable(var))
             string += ',is_cuda={})'.format(is_cuda(var))
-        if fp is not None and fp >= 0:
-            new_v = mt.round_list(var.tolist(), fp=fp)
-        else:
-            new_v = var.tolist()
-        data_str_raw = str(new_v).replace('\'', '')
-        string += mt.get_data_str(data_str_raw, chars)
+
+        if chars != 0:  # -1 for all, or x>0 for x chars
+            # new_v = var.tolist()
+            if fp >= 0 and len(var.size()) > 0:
+                if all((mt.is_int(item) or mt.is_float(item)) for item in var):  # 1d list of int and floats
+                    if all(mt.is_int(item) for item in var):  # if all ints - no rounding
+                        f_format = '{:,}'
+                    else:
+                        f_format = '{:,.%xf}' % fp
+                    new_v = [f_format.format(li_item) for li_item in var]
+                else:  # >1d list or 1d with not just ints and floats
+                    new_v = np.around(var.tolist(), fp).tolist()
+            else:
+                new_v = var
+            data_str_raw = str(new_v).replace('\'', '')
+            string += mt.get_data_str(data_str_raw, chars)
 
         if rec and len(var.size()) > 0:  # recursive call
-            inner_str = to_str(var=var[0], title='{}[0]'.format(title), chars=chars, fp=fp, wm=wm, rec=rec)
+            inner_str = to_str(var=var[0], title='{}[0]'.format(title), chars=chars, fp=fp, rec=rec)
             string += '\n\t{}'.format(inner_str)
-
     else:  # if it's not Tensor - use the default to_str() from misc_tools
         string = mt.to_str(var, title=title, chars=chars, wm=wm, fp=fp, rec=rec)
     return string
@@ -548,7 +559,7 @@ def get_torch_version(ack: bool = False, tabs: int = 1) -> str:
     """
     # noinspection PyUnresolvedReferences
     string = mt.add_color('{}* PyTorch Version {}'.format(tabs * '\t', torch.__version__), ops=mt.SUCCESS_C)
-    string += mt.add_color(' - cuda detected ? ', ops=mt.SUCCESS_C)
+    string += mt.add_color(' - GPU detected ? ', ops=mt.SUCCESS_C)
     if cuda_on():
         string += mt.add_color('True', ops=mt.SUCCESS_C2)
     else:
